#ifndef _WEBCAMINTERFACE_
#define _WEBCAMINTERFACE_

#include <iostream>
#include <vector>

#include "Matrix.h"

#include <cv.h>
#include <cvcam.h>
#include <highgui.h>
#include <math.h>
#include <time.h>

#define POINT_COUNT ( 1000 )
#define MOTION_BUFFER_SIZE ( 3 )

/**
 *  Webcaminterface osztly
 *
 *  Webcamera mozganak meghatarozasa a rogzitett kepekbol.
 *  Az osztaly tarolja az elozo, es jelenlegi kepkockakbol kinyert adatokat.
 *  Az eredmenyt, egy 4 elemu vectorban adja meg, mely az eltolas, a skalazas
 *  es a forgatas erteket tartalmazza.
 */
 
#define SOURCE_TYPE_CAM ( 0 )
#define SOURCE_TYPE_AVI ( 1 )

#define FRAME_WIDTH ( 160 )
#define FRAME_HEIGHT ( 120 )

class WebcamInterface {
        private:
                CvCapture * cvCapture;                  /**< OpenCV kepforras */
                int sourceType;

                IplImage * frame;                       /**< Webcam-bol nyert kep */
                IplImage * grayFrame;

                IplImage * image, * pyramid;            /**< A regi kep */
                IplImage * newImage, * newPyramid;      /**< Az uj-jelenlegi kep es segedkep hozza */
                IplImage * eig, * temp;                 /**< segedkepek a sarokpontdetektalashoz */

                CvPoint2D32f * points, * newPoints;     /**< Az elozo es a jelenlegi kepen detektalt pontok */
                char * status;                          /**< A pontokat felhasznalhatosagat jelolo terkep */

                int of_flags;                           /**< Az OpenCV optical flow algoritmusanak beallitasai */

                std::vector< std::vector<float> > shifts;


        public:
                WebcamInterface( int camID = 0 );
                WebcamInterface( char * aviFileName );
                ~WebcamInterface( );

                void histogramEqualization( IplImage * img );

                void init( );                           /**< Memoriafoglalasok, elso kep felvetele */
                std::vector<float> getMotion( void );   /**< Szurt mozgas */
                std::vector<float> getShift( void );    /**< Ket kep kozotti mozgas kiszamitasa */
                void RANSAC( int n );                   /**< Outlier pontok kiszurese */

                IplImage * getImage( void );            /**< webcam kep lekerdezese */
};

#endif /* _WEBCAMINTERFACE_ */
