/**
 *  Visual Motion Based Human-Computer Interface
 *
 *  Webcamera mozgatasaval iranyitott Kepnezegeto program.
 *  A kep nevet parameterben lehet megadni.
 *
 */

#include <stdio.h>
#include <time.h>

#include <iostream>
#include <vector>
#include <string>

using namespace std;


#include "WebcamInterface.h"

using namespace std;



int main ( int argc, char ** argv )
{
        WebcamInterface * wci;                  /** Maga az interface */
        IplImage        * image=0,              /** A megjelenitendo kep */
                        * temp = 0,             /** Erre rajzoljuk a teglalapot */
                        * subImage = 0,         /** A kep kivagasa */
                        * rotated = 0,          /** A kivagott kep elforgatva */
                        * frame = 0;            /** Webkamera (vagy AVI) kep */
                
        float x, y, z, r, maxz;                 /** Ablak pozicioja, merete */
        int rotEnabled;                         /** Engedelyezve van-e az elforgats */
        int frameNum, startTime;
                                
        CvFont font;

        cvInitFont(&font,CV_FONT_HERSHEY_TRIPLEX|CV_FONT_ITALIC, 1.0, 1.0,0, 1);

        

        /** Paramterek ellenrzse */  
        if ( argc < 2 ) {
                std::cout << "Hasznalat: neye kepfajl [AVI fajl]" << std::endl;
                
                return -1;
        }

        /** Interface indtsa */
        if( argc == 2 )
                wci = new WebcamInterface( );           /**< webcamerval */
        else
                wci = new WebcamInterface( argv[2] );   /**< AVI fajllal */

        wci->init( );

        /** Ablakok ltrehozsa */
        cvNamedWindow( "image", 0 );
        cvNamedWindow( "subImage", 0 );
        cvNamedWindow( "webcam", 0 );

        /** Kpek inicializlsa */
        image = cvLoadImage( argv[1], 1 );
        temp = cvCreateImage( cvGetSize(image), 8, 3 );

        /** A teglalap kezdeti pozicioja, merete */
        x = image->width/2; y = image->height/2; r = 0.0f;
        maxz = image->width < image->height ? image->width/2-10 : image->height/2-10;
        z = maxz/2;

        /** CV transzformcis mtrix a forgatshoz */
        float m[6];
        CvMat rotation = cvMat( 2, 3, CV_32F, m );
        CvPoint2D32f center;

        frameNum = 0;
        startTime = clock( );

        rotEnabled = 0;

        char c = 0;
        while ( c != 'x' ) {
        
            
                /** Kpre a teglalap rarajzolasa, megjelenitese */
                cvCopy( image, temp, 0 );
                cvRectangle( temp, cvPoint( int(x)-int(z), int(y)-int(z) ), cvPoint(int(x)+int(z), int(y)+int(z)), CV_RGB( 255, 255, 255 ), 3, 8, 0 );
                cvShowImage( "image", temp );

                /**
                 *  Kepreszlet kivagasa, forgatasa, megjelenitese
                 *  Csak akkor foglalunk ujra memoriat, ha szukseges
                 *  Az elforgatast is csak akkor, ha szukseges
                 */
                
                if ( !subImage ) {
                        subImage = cvCreateImage( cvSize(int(z)*2, int(z)*2), 8, 3 );
                        rotated = cvCreateImage( cvGetSize(subImage), 8, 3 );
                }
                else {
                        if ( subImage->width != int(z)*2 ) {
                                cvReleaseImage( &subImage );
                                subImage = cvCreateImage( cvSize(int(z)*2, int(z)*2), 8, 3 );
                        }
                        if( rotEnabled && subImage->width != rotated->width ) {
                                cvReleaseImage( &rotated );
                                rotated = cvCreateImage( cvGetSize(subImage), 8, 3 );
                        }
                }
                cvGetRectSubPix( image, subImage, cvPoint2D32f( x, y ) );
                if ( rotEnabled ) {
                        center.x = z; center.y = z;
                        cv2DRotationMatrix( center, r, 1.0, &rotation );
                        cvWarpAffine( subImage, rotated, &rotation );

                        cvShowImage( "subImage", rotated );
                } else
                        cvShowImage( "subImage", subImage );

                /** Mozgasinformacio lekerdezese */
                std::vector<float> motion = wci->getMotion( );

                /** Uj poziciok, meret beallitasa */
                x += -motion[0];
                y +=  motion[1];
                z += (motion[2]-1.0f)*200;//frame->width * 200 * 200;
                r -= motion[3]*57.0f;

                if ( z < 10 ) z = 10;
                if ( z > maxz ) z = maxz;

                if ( r < -45 ) r = -45;
                if ( r > 45 ) r = 45;

                if( x < z ) x = z;
                if( y < z ) y = z;

                if ( x > image->width-1-z ) x = image->width-1-z;
                if ( y > image->height-1-z ) y = image->height-1-z;

                /** Billentyu lekerese */
                c = cvWaitKey( 5 );
                if( c == 'x' ) break;
                switch( (char) c ) {
                        case 's': case 'S':
                                x = image->width/2; y = image->height/2; r = 0.0f;
                                z = maxz/2;
                                break;
                        case '+':
                                z -= 5;
                                if ( z < 10 ) z = 10;

                                break;
                        case '-':
                                z += 5;
                                if ( z > maxz ) z = maxz;
                                break;
                        case 'r': case 'R':
                                rotEnabled = 1-rotEnabled;
                                r = 0.0f;
                        default:
                                break;
                }
                
                /** Sebesseginformaciok kiirasa */
                frameNum++;
                
                float fps = (float)frameNum / ( (float) (clock()-startTime) / CLOCKS_PER_SEC);
                char fpsstr[30]; sprintf( fpsstr, "%lf",  fps );
                std::string title = "FPS: " + std::string( fpsstr );
 
                /** Vezrl kp lekrdezse, megjelentse */
                frame = wci->getImage( );
                cvPutText (frame,title.c_str(),cvPoint(0,0), &font, cvScalar(20,20,200));
                cvShowImage( "webcam", frame );

        }

        cvDestroyWindow("image");
        cvDestroyWindow("subImage");
        cvDestroyWindow("webcam");

        delete wci;

        return 0;
}
